import { Host, h, } from "@stencil/core";
import debounce from "lodash/debounce";
import { getBundle } from "../../apis/commercelayer/bundles/index";
import { addItem, getCart, getCartQuantity } from "../../apis/commercelayer/cart";
import { getSku } from "../../apis/commercelayer/skus/index";
import { listenTo } from "../../apis/event";
import { log } from "../../utils/logger";
import { isValidCode, isValidQuantity, logCode, logUnion, unionToTuple, } from "../../utils/validation-helpers";
export class ClAddToCart {
    host;
    kindList = unionToTuple()("sku", "bundle");
    kindDefault = "sku";
    /**
     * Indicates whether the code refers to a `sku` or a `bundle`.
     * @default sku
     */
    kind = "sku";
    /**
     * The SKU or bundle code (i.e. the unique identifier of the product or bundle you want to add to the shopping cart).
     */
    code;
    /**
     * The number of units of the selected product you want to add to the shopping cart.
     * @default 1
     */
    quantity = 1;
    /**
     * A custom name for the product or bundle that will be added to the cart.
     * If not provided, the name will be taken from the item being added.
     */
    name;
    /**
     * A custom image URL for the product or bundle that will be added to the cart.
     * If not provided, the image URL will be taken from the item being added.
     */
    imageUrl;
    /**
     * The frequency which generates a [subscription](https://docs.commercelayer.io/core/v/how-tos/placing-orders/subscriptions). The value must be supported by the associated subscription model.
     */
    frequency;
    /**
     * The item is currently being added to the cart.
     * @default false
     */
    busy = false;
    inventory;
    cart;
    async componentWillLoad() {
        await this.updateSku(this.code);
        await this.updateQuantity(this.quantity);
        this.cart = (await getCart()) ?? undefined;
        listenTo("cl-cart-update", (event) => {
            const [kind, code] = event.detail.request.args;
            this.cart = event.detail.response;
            if (kind === this.kind && code === this.code) {
                this.busy = false;
            }
        });
        listenTo("cl-cart-hostedcartupdate", (event) => {
            this.cart = event.detail.response;
        });
    }
    async watchKindHandler(newValue) {
        if (newValue == null) {
            this.kind = this.kindDefault;
            return;
        }
        logUnion(this.host, "kind", newValue, this.kindList);
    }
    async watchCodeHandler(newValue) {
        await this.debouncedUpdateSku(newValue);
    }
    async watchQuantityHandler(newValue) {
        await this.updateQuantity(newValue);
    }
    updateSku = async (code) => {
        logCode(this.host, code);
        if (isValidCode(code)) {
            switch (this.kind) {
                case "bundle":
                    this.inventory = (await getBundle(code))?.inventory;
                    if (this.inventory === undefined) {
                        log("warn", `Cannot find code ${code}.`, this.host);
                    }
                    break;
                default:
                    this.inventory = (await getSku(code))?.inventory;
                    if (this.inventory === undefined) {
                        log("warn", `Cannot find code ${code}.`, this.host);
                    }
                    break;
            }
        }
    };
    debouncedUpdateSku = debounce(this.updateSku, 10);
    async updateQuantity(quantity) {
        if (!isValidQuantity(quantity)) {
            this.quantity = 0;
        }
    }
    handleKeyPress(event) {
        if (event.key === "Enter" || event.key === " ") {
            this.handleAddItem();
        }
    }
    handleAddItem() {
        if (this.code !== undefined && this.canBeSold() && !this.busy) {
            this.busy = true;
            addItem(this.kind ?? this.kindDefault, this.code, this.quantity, {
                name: this.name,
                image_url: this.imageUrl,
                frequency: this.frequency,
            }).catch((error) => {
                this.busy = false;
                throw error;
            });
        }
    }
    /**
     * Check whether the sku is soldable.
     * @returns Returns true when item is soldable.
     */
    canBeSold() {
        const cartQuantity = getCartQuantity(this.cart, this.kind, this.code);
        const hasQuantity = this.inventory?.quantity === undefined ||
            this.quantity <= this.inventory.quantity - cartQuantity;
        return (isValidCode(this.code) &&
            this.quantity > 0 &&
            this.inventory?.available === true &&
            hasQuantity);
    }
    render() {
        return (h(Host, { key: '0f5954bd2e03b8c13718e59a516cd7cc625e1df3', role: "button", tabindex: "0", "aria-disabled": this.canBeSold() && !this.busy ? undefined : "true", "aria-busy": this.busy ? "true" : undefined, onKeyPress: (event) => {
                this.handleKeyPress(event);
            }, onClick: () => {
                this.handleAddItem();
            } }, h("slot", { key: 'f248b31690ef9a04c8ae471db179f210755d1ab1' })));
    }
    static get is() { return "cl-add-to-cart"; }
    static get encapsulation() { return "shadow"; }
    static get properties() {
        return {
            "kind": {
                "type": "string",
                "attribute": "kind",
                "mutable": true,
                "complexType": {
                    "original": "\"sku\" | \"bundle\"",
                    "resolved": "\"bundle\" | \"sku\" | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "default",
                            "text": "sku"
                        }],
                    "text": "Indicates whether the code refers to a `sku` or a `bundle`."
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "\"sku\""
            },
            "code": {
                "type": "string",
                "attribute": "code",
                "mutable": false,
                "complexType": {
                    "original": "string | undefined",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": true,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The SKU or bundle code (i.e. the unique identifier of the product or bundle you want to add to the shopping cart)."
                },
                "getter": false,
                "setter": false,
                "reflect": true
            },
            "quantity": {
                "type": "number",
                "attribute": "quantity",
                "mutable": true,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "default",
                            "text": "1"
                        }],
                    "text": "The number of units of the selected product you want to add to the shopping cart."
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "1"
            },
            "name": {
                "type": "string",
                "attribute": "name",
                "mutable": false,
                "complexType": {
                    "original": "string | undefined",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "A custom name for the product or bundle that will be added to the cart.\nIf not provided, the name will be taken from the item being added."
                },
                "getter": false,
                "setter": false,
                "reflect": true
            },
            "imageUrl": {
                "type": "string",
                "attribute": "image-url",
                "mutable": false,
                "complexType": {
                    "original": "string | undefined",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "A custom image URL for the product or bundle that will be added to the cart.\nIf not provided, the image URL will be taken from the item being added."
                },
                "getter": false,
                "setter": false,
                "reflect": true
            },
            "frequency": {
                "type": "string",
                "attribute": "frequency",
                "mutable": false,
                "complexType": {
                    "original": "string | undefined",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The frequency which generates a [subscription](https://docs.commercelayer.io/core/v/how-tos/placing-orders/subscriptions). The value must be supported by the associated subscription model."
                },
                "getter": false,
                "setter": false,
                "reflect": true
            }
        };
    }
    static get states() {
        return {
            "busy": {},
            "inventory": {},
            "cart": {}
        };
    }
    static get elementRef() { return "host"; }
    static get watchers() {
        return [{
                "propName": "kind",
                "methodName": "watchKindHandler"
            }, {
                "propName": "code",
                "methodName": "watchCodeHandler"
            }, {
                "propName": "quantity",
                "methodName": "watchQuantityHandler"
            }];
    }
}
