import { h, } from "@stencil/core";
import debounce from "lodash/debounce";
import { getCart, getCartQuantity } from "../../apis/commercelayer/cart";
import { getSku } from "../../apis/commercelayer/skus/index";
import { listenTo } from "../../apis/event";
import { isValidCode, logCode, logUnion, unionToTuple, } from "../../utils/validation-helpers";
export class ClAvailability {
    host;
    kindList = unionToTuple()("sku", "bundle");
    kindDefault = "sku";
    /**
     * Indicates whether the code refers to a `sku` or a `bundle`.
     *
     * _⚠️ `bundle` is not fully implemented._
     *
     * @default sku
     */
    kind = "sku";
    /**
     * The SKU or bundle code (i.e. the unique identifier of the product or bundle whose availability you want to display).
     */
    code;
    /**
     * The rule used to determine the information that will be displayed.
     * `cheapest` is the delivery lead time associated with the lower shipping method cost,
     * `fastest` is the delivery lead time associated with the lower average time to delivery.
     */
    rule = "cheapest";
    cart;
    async componentWillLoad() {
        logCode(this.host, this.code);
        this.cart = (await getCart()) ?? undefined;
        await this.updateAvailability(this.kind, this.code);
        listenTo("cl-cart-update", (event) => {
            this.cart = event.detail.response;
            void this.updateAvailability(this.kind, this.code);
        });
        listenTo("cl-cart-hostedcartupdate", (event) => {
            this.cart = event.detail.response;
            void this.updateAvailability(this.kind, this.code);
        });
    }
    async watchKindHandler(newValue) {
        if (newValue == null) {
            this.kind = this.kindDefault;
            return;
        }
        logUnion(this.host, "kind", newValue, this.kindList);
        await this.debouncedUpdateAvailability(newValue, this.code);
    }
    async watchPropHandler(newValue) {
        logCode(this.host, newValue);
        await this.debouncedUpdateAvailability(this.kind, newValue);
    }
    updateAvailability = async (kind, code) => {
        let sku;
        if (kind !== "bundle" && isValidCode(code)) {
            sku = await getSku(code);
        }
        this.host
            .querySelectorAll("cl-availability-status, cl-availability-info")
            .forEach((element) => {
            element.dispatchEvent(new CustomEvent("availabilityUpdate", {
                detail: {
                    sku,
                    rule: this.rule,
                    cartQuantity: getCartQuantity(this.cart, kind, code),
                },
            }));
        });
    };
    debouncedUpdateAvailability = debounce(this.updateAvailability, 10);
    render() {
        return h("slot", { key: '50f44a10f705b608676b3d7963ffe6c3046174a9' });
    }
    static get is() { return "cl-availability"; }
    static get encapsulation() { return "shadow"; }
    static get properties() {
        return {
            "kind": {
                "type": "string",
                "attribute": "kind",
                "mutable": true,
                "complexType": {
                    "original": "\"sku\" | \"bundle\"",
                    "resolved": "\"bundle\" | \"sku\" | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "default",
                            "text": "sku"
                        }],
                    "text": "Indicates whether the code refers to a `sku` or a `bundle`.\n\n_\u26A0\uFE0F `bundle` is not fully implemented._"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "\"sku\""
            },
            "code": {
                "type": "string",
                "attribute": "code",
                "mutable": false,
                "complexType": {
                    "original": "string | undefined",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": true,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The SKU or bundle code (i.e. the unique identifier of the product or bundle whose availability you want to display)."
                },
                "getter": false,
                "setter": false,
                "reflect": true
            },
            "rule": {
                "type": "string",
                "attribute": "rule",
                "mutable": false,
                "complexType": {
                    "original": "\"cheapest\" | \"fastest\"",
                    "resolved": "\"cheapest\" | \"fastest\"",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The rule used to determine the information that will be displayed.\n`cheapest` is the delivery lead time associated with the lower shipping method cost,\n`fastest` is the delivery lead time associated with the lower average time to delivery."
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "\"cheapest\""
            }
        };
    }
    static get states() {
        return {
            "cart": {}
        };
    }
    static get elementRef() { return "host"; }
    static get watchers() {
        return [{
                "propName": "kind",
                "methodName": "watchKindHandler"
            }, {
                "propName": "code",
                "methodName": "watchPropHandler"
            }];
    }
}
