import { Host, h, } from "@stencil/core";
import { iframeResizer } from "iframe-resizer";
import { getCartUrl, isValidUrl, triggerHostedCartUpdate, updateCartUrl, } from "../../apis/commercelayer/cart";
import { listenTo } from "../../apis/event";
import { getClosestLocationHref } from "../../utils/url";
const hostedCartIframeUpdateEvent = { type: "update" };
export class ClCart {
    host;
    iframe;
    /**
     * By default the `cl-cart` is directly displayed in-place.
     * Setting the `type` to `mini` will change the behavior to be a minicart.
     */
    type;
    /**
     * If `true` the minicart automatically opens as soon as an item is added to the shopping cart
     * (available _only_ when the `cl-cart` component is used as _minicart_).
     */
    openOnAdd = false;
    /**
     * Indicate whether the minicart is open or not
     * (available _only_ when the `cl-cart` component is used as _minicart_).
     */
    open = false;
    /** Current hosted cart url */
    href;
    isMinicart = false;
    /**
     * Used for:
     * 1. As query parameter that re-open the minicart when clicking on `< Return to cart` (Hosted Cart link).
     * 2. As body class when the minicart is open to disable page scrolling.
     */
    openDirective = "cl-cart--open";
    flag_listenForHostedCartUpdateResponse = true;
    async componentWillLoad() {
        listenTo("cl-cart-hostedcartupdate", (event) => {
            const [iframeId] = event.detail.request.args;
            if (this.iframe.id !== iframeId) {
                this.flag_listenForHostedCartUpdateResponse = false;
                this.iframe.iFrameResizer.sendMessage(hostedCartIframeUpdateEvent);
            }
        });
        listenTo("cl-cart-update", async () => {
            this.iframe.iFrameResizer.sendMessage(hostedCartIframeUpdateEvent);
            await this.updateUrl(this.openOnAdd);
            if (this.type === "mini" && this.openOnAdd) {
                this.open = true;
            }
        });
        await updateCartUrl(this.getCartPageUrl());
        if (this.checkLocationHrefForOpenDirective()) {
            this.open = true;
        }
        this.updateMinicartUrl();
    }
    async updateUrl(bypassMinicartCheck) {
        const shouldUpdate = this.href === undefined || !(await isValidUrl(this.href));
        if (((this.type === "mini" && this.open) ||
            this.type !== "mini" ||
            bypassMinicartCheck === true) &&
            shouldUpdate) {
            this.href = await getCartUrl();
        }
    }
    /**
     * Get the current page url.
     * When the component is rendered as `minicart` the cart page url will be the current page url, plus a dedicated query parameter.
     * @returns Current page url ( + query parameter when rendered as `minicart` )
     */
    getCartPageUrl() {
        const closestLocationHref = getClosestLocationHref();
        if (this.type === "mini") {
            const url = new URL(closestLocationHref);
            if (!url.searchParams.has(this.openDirective)) {
                url.searchParams.append(this.openDirective, "");
            }
            return url.href;
        }
        return closestLocationHref;
    }
    /**
     * Check whether the current url has the `openDirective` query parameter.
     * @returns Whether the current url has the `openDirective`
     */
    checkLocationHrefForOpenDirective() {
        const url = new URL(location.href);
        if (this.type === "mini" && url.searchParams.has(this.openDirective)) {
            url.searchParams.delete(this.openDirective);
            history.replaceState({}, "", url.href);
            return true;
        }
        return false;
    }
    updateMinicartUrl() {
        void this.updateUrl();
        if (this.type === "mini") {
            document.body.classList.toggle(this.openDirective, this.open);
        }
    }
    watchOpenHandler(opened) {
        if (this.type === "mini") {
            this.updateMinicartUrl();
            if (!opened) {
                this.host.closest("cl-cart-link")?.focus();
            }
        }
    }
    componentDidLoad() {
        const onMessage = (data) => {
            switch (data.message.type) {
                case "update":
                    if (this.flag_listenForHostedCartUpdateResponse) {
                        void triggerHostedCartUpdate(this.iframe.id, data.message.payload ?? null);
                    }
                    this.flag_listenForHostedCartUpdateResponse = true;
                    break;
                case "close":
                    if (this.type === "mini") {
                        this.open = false;
                    }
                    break;
                case "blur":
                    if (this.type === "mini" && this.open) {
                        this.iframe.focus();
                    }
                    break;
            }
        };
        iframeResizer({
            checkOrigin: false,
            onMessage,
        }, this.iframe);
    }
    handleKeyDown(event) {
        if (this.type === "mini" && event.key === "Escape" && this.open) {
            this.handleCloseMinicart(event);
        }
    }
    handleCloseMinicart(event) {
        event.stopPropagation();
        this.open = false;
    }
    render() {
        return (h(Host, { key: '79c5e475c28b6d116aebed79a8aa4372e7aa1801', ...(this.type === "mini"
                ? {
                    role: this.open ? "alertdialog" : undefined,
                    "aria-modal": this.open ? "true" : undefined,
                    "aria-hidden": !this.open ? "true" : undefined,
                    tabindex: !this.open ? "-1" : undefined,
                    onClick: (event) => {
                        if (event.offsetX < 0 || event.offsetY < 0) {
                            this.handleCloseMinicart(event);
                        }
                    },
                }
                : {}) }, h("div", { key: '2d935c3ea86f4a7568b28ca31f6a02fb1e085af6', part: "container" }, this.type === "mini" ? (h("button", { type: "button", "aria-label": "Close", part: "close-button", onClick: (event) => {
                this.handleCloseMinicart(event);
            } }, "Close")) : null, h("iframe", { key: 'ab2c3e432dbd3b5884c6eb4c04aacd46b72b0b4f', part: "iframe", title: "My Cart", allow: "payment", ref: (el) => {
                this.iframe = el;
            }, src: this.href, style: {
                width: "1px",
                "min-width": "100%",
                "min-height": "100%",
                border: "none",
            } }))));
    }
    static get is() { return "cl-cart"; }
    static get encapsulation() { return "shadow"; }
    static get styles() { return ":host([type='mini']) {\n      display: none;\n    }\n\n    :host > div {\n      overflow: auto;\n      width: 100%;\n      display: flex;\n    }"; }
    static get properties() {
        return {
            "type": {
                "type": "string",
                "attribute": "type",
                "mutable": false,
                "complexType": {
                    "original": "\"mini\" | undefined",
                    "resolved": "\"mini\" | undefined",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "By default the `cl-cart` is directly displayed in-place.\nSetting the `type` to `mini` will change the behavior to be a minicart."
                },
                "getter": false,
                "setter": false,
                "reflect": true
            },
            "openOnAdd": {
                "type": "boolean",
                "attribute": "open-on-add",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "If `true` the minicart automatically opens as soon as an item is added to the shopping cart\n(available _only_ when the `cl-cart` component is used as _minicart_)."
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "false"
            },
            "open": {
                "type": "boolean",
                "attribute": "open",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Indicate whether the minicart is open or not\n(available _only_ when the `cl-cart` component is used as _minicart_)."
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "false"
            }
        };
    }
    static get states() {
        return {
            "href": {},
            "isMinicart": {}
        };
    }
    static get elementRef() { return "host"; }
    static get watchers() {
        return [{
                "propName": "open",
                "methodName": "watchOpenHandler"
            }];
    }
    static get listeners() {
        return [{
                "name": "keydown",
                "method": "handleKeyDown",
                "target": "window",
                "capture": false,
                "passive": false
            }];
    }
}
