import debounce from "lodash/debounce";
import memoize from "lodash/memoize";
export const pDebounce = (input, options) => {
    const incrementalArgs = [];
    const incrementalResolve = [];
    const fn = async (resolves) => {
        const result = await input(incrementalArgs);
        resolves.forEach((resolve) => {
            resolve(result);
        });
        incrementalResolve.length = 0;
        incrementalArgs.length = 0;
    };
    const debounced = debounce(fn, options?.wait, { maxWait: options?.maxWait });
    return async (item) => {
        if (item !== undefined) {
            incrementalArgs.push(...item);
        }
        return await new Promise((resolve) => {
            incrementalResolve.push(resolve);
            void debounced(incrementalResolve);
        });
    };
};
/**
 * Creates a memoized-debounced function that delays invoking `func` until after `wait` milliseconds
 * have elapsed since the last time the memoized-debounced function was invoked.
 *
 * The function is also **memoized** using all arguments as the map cache key. You can override this
 * default by providing a `resolver` function.
 * @param func The function to debounce.
 * @param wait The number of milliseconds to delay.
 * @param options The options object.
 * @param resolver The function to resolve the cache key (for memoized function).
 * @returns Returns the new memoized function debounced.
 */
export function memoDebounce(func, wait = 0, options = {}, resolver = (...args) => JSON.stringify(args)) {
    const debounceMemo = memoize((..._args) => debounce(func, wait, options), resolver);
    function wrappedFunction(...args) {
        return debounceMemo(...args)(...args);
    }
    wrappedFunction.flush = (...args) => {
        return debounceMemo(...args).flush();
    };
    wrappedFunction.cancel = (...args) => {
        debounceMemo(...args).cancel();
    };
    return wrappedFunction;
}
