import { a as memoize, p as pDebounce, g as getConfig, c as createClient, b as getAccessToken, i, d as api, e as getKeyForCart, f as fireEvent, h as getOrganizationConfig } from './config.js';

/**
 * Create a draft order.
 * @see https://docs.commercelayer.io/core/v/how-tos/shopping-cart/create-a-shopping-cart
 * @returns Returns the created draft order.
 */
async function createEmptyCart() {
    const config = getConfig();
    const client = await createClient(config);
    const token = await getAccessToken(config);
    const order = await client.orders.create(config.defaultAttributes?.orders ?? {});
    if (token.type === "guest") {
        setCartId(order.id);
    }
    await triggerCartUpdate();
    return order;
}
const LINE_ITEMS_SHIPPABLE = ["skus", "bundles"];
const LINE_ITEMS_SHOPPABLE = [...LINE_ITEMS_SHIPPABLE, "gift_cards"];
function isValidForCheckout(order) {
    return (order.line_items?.find((lineItem) => {
        return LINE_ITEMS_SHOPPABLE.includes(lineItem.item_type);
    }) !== undefined);
}
function removeCartId() {
    const config = getConfig();
    api.remove(getKeyForCart(config));
}
function setCartId(cartId) {
    const config = getConfig();
    api.set(getKeyForCart(config), cartId);
}
function getCartId() {
    const config = getConfig();
    return api.get(getKeyForCart(config)) ?? null;
}
async function isValidUrl(url) {
    const cartId = (await getCart())?.id;
    return cartId != null && url.includes(`/${cartId}?`);
}
/**
 * Get the Hosted Cart url.
 * @param forceCartToExist When true it will create an empty cart if not existing before.
 * @returns Returns the Hosted Cart url.
 */
async function getCartUrl(forceCartToExist = false) {
    let cartId = (await getCart())?.id;
    if (cartId === undefined && forceCartToExist) {
        const cart = await createEmptyCart();
        cartId = cart.id;
    }
    const organizationConfig = await getOrganizationConfig({
        orderId: cartId ?? "null",
    });
    return organizationConfig.links.cart;
}
async function getCheckoutUrl() {
    const cart = await getCart();
    if (cart === null || !isValidForCheckout(cart)) {
        return undefined;
    }
    const organizationConfig = await getOrganizationConfig({
        orderId: cart.id ?? "null",
    });
    return organizationConfig.links.checkout;
}
/**
 * Update the language code of the order when it is different from the drop-in language code configuration.
 */
async function updateOrderLanguage(order, orderParams) {
    const config = getConfig();
    const client = await createClient(config);
    if (order === null) {
        return null;
    }
    if (order.language_code === config.defaultAttributes?.orders?.language_code) {
        return order;
    }
    const updatedOrder = await client.orders.update({
        id: order.id,
        language_code: config.defaultAttributes?.orders?.language_code,
    }, orderParams);
    return updatedOrder;
}
async function _getCart() {
    const config = getConfig();
    const client = await createClient(config);
    const token = await getAccessToken(config);
    const orderParams = {
        include: ["line_items.item", "line_items.line_item_options.sku_option"],
    };
    if (token.type === "guest") {
        const orderId = getCartId();
        if (orderId === null) {
            return null;
        }
        const order = await client.orders
            .retrieve(orderId, orderParams)
            .catch(() => null);
        if (order?.editable === false) {
            removeCartId();
            return null;
        }
        return await updateOrderLanguage(order, orderParams);
    }
    const jwt = i(token.accessToken);
    if (!("market" in jwt.payload) || jwt.payload.market?.id == null) {
        return null;
    }
    const [order = null] = await client.customers.orders(token.customerId, {
        ...orderParams,
        filters: {
            market_id_in: jwt.payload.market.id.join(","),
            guest_false: true,
            editable_true: true,
            status_eq: "pending",
        },
        sort: {
            updated_at: "desc",
        },
        pageSize: 1,
    });
    return await updateOrderLanguage(order, orderParams);
}
const getCart = memoize(pDebounce(_getCart, { wait: 10, maxWait: 50 }));
/**
 * Trigger the `cartUpdate` event.
 */
const triggerCartUpdate = async (...args) => {
    const order = await getCart();
    if (order !== null) {
        fireEvent("cl-cart-update", args, order);
    }
    return order;
};
/**
 * Trigger the `hostedCartUpdate` event.
 * @param iframeId iFrame ID who triggered the event
 */
const triggerHostedCartUpdate = async (iframeId, order) => {
    if (order !== null) {
        fireEvent("cl-cart-hostedcartupdate", [iframeId, order], order);
    }
    return order;
};
const addItem = async (kind, code, quantity, options = {}) => {
    const client = await createClient(getConfig());
    const orderId = (await getCart())?.id ?? (await createEmptyCart()).id;
    const lineItem = await client.line_items.create({
        ...options,
        order: {
            id: orderId,
            type: "orders",
        },
        quantity,
        ...(kind === "sku" ? { sku_code: code } : { bundle_code: code }),
        _update_quantity: true,
    });
    fireEvent("cl-cart-additem", [kind, code, quantity, options], lineItem);
    if (getCart.cache.clear !== undefined) {
        getCart.cache.clear();
    }
    await triggerCartUpdate(kind, code, quantity, options);
    return lineItem;
};
/**
 * Update the `cart_url` property of the cart on Commerce Layer.
 * `cart_url` is used in the Hosted Checkout as a link for "< Return to cart".
 * @param cartUrl new cart_url
 */
async function updateCartUrl(cartUrl) {
    const cart = await getCart();
    if (cart !== null && cart.cart_url !== cartUrl) {
        const client = await createClient(getConfig());
        await client.orders.update({
            id: cart.id,
            cart_url: cartUrl,
        });
    }
}
/**
 * Get quantity of a specific item in the cart, by reading the `line_items`.
 */
function getCartQuantity(cart, kind, code) {
    return (cart?.line_items
        ?.filter((item) => {
        switch (kind) {
            case "bundle":
                return item.bundle_code === code;
            default:
                return item.sku_code === code;
        }
    })
        .map((item) => item.quantity)
        .reduce((acc, val) => acc + val, 0) ?? 0);
}

export { getCartQuantity as a, getCartUrl as b, getCheckoutUrl as c, addItem as d, getCart as g, isValidUrl as i, triggerHostedCartUpdate as t, updateCartUrl as u };
