import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import { m as memoize, f as fireEvent, c as createClient, g as getConfig, l as listenTo, k as debounce } from './config.js';
import { _ as _getBundleViaList } from './list2.js';
import { g as getCart, d as addItem, a as getCartQuantity } from './cart.js';
import './list.js';
import { g as getSku } from './skus.js';
import { u as unionToTuple, b as logUnion, a as logCode, i as isValidCode, d as log, e as isValidQuantity } from './validation-helpers.js';

function getBundleInventory(bundle) {
    return bundle.sku_list?.sku_list_items?.reduce((inventory, skuListItem) => {
        const quantity = skuListItem.sku?.inventory?.quantity == null
            ? undefined
            : Math.floor(skuListItem.sku?.inventory?.quantity /
                (skuListItem.quantity ?? 1));
        const available = skuListItem.sku.inventory?.available ?? false;
        return {
            available: (inventory.available ?? true) && available,
            levels: inventory.levels,
            quantity: quantity != null &&
                quantity < (inventory.quantity ?? Number.POSITIVE_INFINITY)
                ? quantity
                : inventory.quantity,
        };
    }, {
        available: true,
        levels: [],
    });
}
const getMemoizedBundle = memoize(async (code) => {
    const { id } = (await _getBundleViaList(code)) ?? {};
    if (id == null) {
        return undefined;
    }
    const client = await createClient(getConfig());
    const bundle = (await client.bundles.retrieve(id, {
        include: ["sku_list.sku_list_items.sku"],
    }));
    return {
        ...bundle,
        inventory: getBundleInventory(bundle),
    };
});
const getBundle = async (code) => {
    const bundle = await getMemoizedBundle(code);
    fireEvent("cl-bundles-getbundle", [code], bundle);
    return bundle;
};

const ClAddToCart$1 = /*@__PURE__*/ proxyCustomElement(class ClAddToCart extends HTMLElement {
    constructor() {
        super();
        this.__registerHost();
        this.__attachShadow();
    }
    get host() { return this; }
    kindList = unionToTuple()("sku", "bundle");
    kindDefault = "sku";
    /**
     * Indicates whether the code refers to a `sku` or a `bundle`.
     * @default sku
     */
    kind = "sku";
    /**
     * The SKU or bundle code (i.e. the unique identifier of the product or bundle you want to add to the shopping cart).
     */
    code;
    /**
     * The number of units of the selected product you want to add to the shopping cart.
     * @default 1
     */
    quantity = 1;
    /**
     * A custom name for the product or bundle that will be added to the cart.
     * If not provided, the name will be taken from the item being added.
     */
    name;
    /**
     * A custom image URL for the product or bundle that will be added to the cart.
     * If not provided, the image URL will be taken from the item being added.
     */
    imageUrl;
    /**
     * The frequency which generates a [subscription](https://docs.commercelayer.io/core/v/how-tos/placing-orders/subscriptions). The value must be supported by the associated subscription model.
     */
    frequency;
    /**
     * The item is currently being added to the cart.
     * @default false
     */
    busy = false;
    inventory;
    cart;
    async componentWillLoad() {
        await this.updateSku(this.code);
        await this.updateQuantity(this.quantity);
        this.cart = (await getCart()) ?? undefined;
        listenTo("cl-cart-update", (event) => {
            const [kind, code] = event.detail.request.args;
            this.cart = event.detail.response;
            if (kind === this.kind && code === this.code) {
                this.busy = false;
            }
        });
        listenTo("cl-cart-hostedcartupdate", (event) => {
            this.cart = event.detail.response;
        });
    }
    async watchKindHandler(newValue) {
        if (newValue == null) {
            this.kind = this.kindDefault;
            return;
        }
        logUnion(this.host, "kind", newValue, this.kindList);
    }
    async watchCodeHandler(newValue) {
        await this.debouncedUpdateSku(newValue);
    }
    async watchQuantityHandler(newValue) {
        await this.updateQuantity(newValue);
    }
    updateSku = async (code) => {
        logCode(this.host, code);
        if (isValidCode(code)) {
            switch (this.kind) {
                case "bundle":
                    this.inventory = (await getBundle(code))?.inventory;
                    if (this.inventory === undefined) {
                        log("warn", `Cannot find code ${code}.`, this.host);
                    }
                    break;
                default:
                    this.inventory = (await getSku(code))?.inventory;
                    if (this.inventory === undefined) {
                        log("warn", `Cannot find code ${code}.`, this.host);
                    }
                    break;
            }
        }
    };
    debouncedUpdateSku = debounce(this.updateSku, 10);
    async updateQuantity(quantity) {
        if (!isValidQuantity(quantity)) {
            this.quantity = 0;
        }
    }
    handleKeyPress(event) {
        if (event.key === "Enter" || event.key === " ") {
            this.handleAddItem();
        }
    }
    handleAddItem() {
        if (this.code !== undefined && this.canBeSold() && !this.busy) {
            this.busy = true;
            addItem(this.kind ?? this.kindDefault, this.code, this.quantity, {
                name: this.name,
                image_url: this.imageUrl,
                frequency: this.frequency,
            }).catch((error) => {
                this.busy = false;
                throw error;
            });
        }
    }
    /**
     * Check whether the sku is soldable.
     * @returns Returns true when item is soldable.
     */
    canBeSold() {
        const cartQuantity = getCartQuantity(this.cart, this.kind, this.code);
        const hasQuantity = this.inventory?.quantity === undefined ||
            this.quantity <= this.inventory.quantity - cartQuantity;
        return (isValidCode(this.code) &&
            this.quantity > 0 &&
            this.inventory?.available === true &&
            hasQuantity);
    }
    render() {
        return (h(Host, { key: '0f5954bd2e03b8c13718e59a516cd7cc625e1df3', role: "button", tabindex: "0", "aria-disabled": this.canBeSold() && !this.busy ? undefined : "true", "aria-busy": this.busy ? "true" : undefined, onKeyPress: (event) => {
                this.handleKeyPress(event);
            }, onClick: () => {
                this.handleAddItem();
            } }, h("slot", { key: 'f248b31690ef9a04c8ae471db179f210755d1ab1' })));
    }
    static get watchers() { return {
        "kind": ["watchKindHandler"],
        "code": ["watchCodeHandler"],
        "quantity": ["watchQuantityHandler"]
    }; }
}, [1, "cl-add-to-cart", {
        "kind": [1537],
        "code": [513],
        "quantity": [1538],
        "name": [513],
        "imageUrl": [513, "image-url"],
        "frequency": [513],
        "busy": [32],
        "inventory": [32],
        "cart": [32]
    }, undefined, {
        "kind": ["watchKindHandler"],
        "code": ["watchCodeHandler"],
        "quantity": ["watchQuantityHandler"]
    }]);
function defineCustomElement$1() {
    if (typeof customElements === "undefined") {
        return;
    }
    const components = ["cl-add-to-cart"];
    components.forEach(tagName => { switch (tagName) {
        case "cl-add-to-cart":
            if (!customElements.get(tagName)) {
                customElements.define(tagName, ClAddToCart$1);
            }
            break;
    } });
}
defineCustomElement$1();

const ClAddToCart = ClAddToCart$1;
const defineCustomElement = defineCustomElement$1;

export { ClAddToCart, defineCustomElement };
