import { p as pDebounce, c as createClient, g as getConfig, u as uniq, j as chunk, m as memoize, f as fireEvent } from './config.js';
import { l as logGroup } from './validation-helpers.js';

const _getPrices = async (skus) => {
    const client = await createClient(getConfig());
    const uniqSkus = uniq(skus);
    const log = logGroup("`getSkuPrices` method invoked with a list of SKU codes");
    log("info", '`getSkuPrices` is the method involved in fetching a list of prices from Commerce Layer. You can follow the request in the "network" panel.');
    log("info", "SKUs", uniqSkus);
    const pageSize = 25;
    const chunkedSkus = chunk(uniqSkus, pageSize);
    const pricesResponse = (await Promise.all(chunkedSkus.map(async (skus) => {
        return await client.prices.list({
            pageSize,
            filters: { sku_code_in: skus.join(",") },
        });
    }))).flat();
    // TODO: this should be used as cache for future calls or to avoid fetching multiple time same items
    const prices = pricesResponse.reduce((prices, price) => {
        if (price.sku_code != null) {
            prices[price.sku_code] = price;
        }
        return prices;
    }, {});
    log.end();
    return prices;
};
const getPrices = pDebounce(_getPrices, { wait: 10, maxWait: 50 });
const getMemoizedPrice = memoize(async (sku) => {
    return await getPrices([sku]).then((result) => result[sku]);
});
const getPrice = async (sku) => {
    const price = await getMemoizedPrice(sku);
    /** @deprecated Use `cl-skus-getprice` instead. This will be removed in a future version. */
    fireEvent("cl-prices-getprice", [sku], price);
    fireEvent("cl-skus-getprice", [sku], price);
    return price;
};

const _getSkusViaList = async (codes) => {
    const client = await createClient(getConfig());
    const uniqCodes = uniq(codes);
    const log = logGroup("`getSkusViaList` method invoked with a list of SKU codes");
    log("info", '`getSkusViaList` is the method involved in fetching a list of SKUs from Commerce Layer. You can follow the request in the "network" panel.');
    log("info", "codes", uniqCodes);
    const pageSize = 25;
    const chunkedCodes = chunk(uniqCodes, pageSize);
    const response = (await Promise.all(chunkedCodes.map(async (codes) => {
        return await client.skus.list({
            pageSize,
            filters: { code_in: codes.join(",") },
            fields: {
                skus: ["id", "code"],
            },
        });
    }))).flat();
    // TODO: this should be used as cache for future calls or to avoid fetching multiple time same items
    const skus = response.reduce((skus, sku) => {
        if (sku.id !== undefined && sku.code !== undefined) {
            skus[sku.code] = sku;
        }
        return skus;
    }, {});
    log.end();
    return skus;
};
const getSkusViaList = pDebounce(_getSkusViaList, { wait: 10, maxWait: 50 });
/**
 * Get a SKU by providing a SKU code using the `list` API.
 * List API **does not provide** _stock information_.
 * @protected
 */
const _getSkuViaList = memoize(async (code) => {
    return await getSkusViaList([code]).then((result) => result[code]);
});

export { _getSkuViaList as _, getPrice as g };
