import { g as getConfig } from './config.js';

/**
 * Outputs a message to the Web console.
 * @param type Type of message.
 * @param messages List of messages.
 */
const log = (type, ...messages) => {
    const { debug } = getConfig();
    if (debug === "all") {
        console[type](...messages);
    }
};
/**
 * The `logGroup()` method creates a new inline group in the Web console log,
 * causing any subsequent console messages to be indented by an additional level,
 * until `log.end()` is called.
 * @param label Label for the group.
 * @param collapsed The new block is collapsed and requires clicking a disclosure button to read it.
 * @returns Returns a function that can be invoked to collect logs. To display collected log you should call `.end()` method.
 * @example
 * const log = logGroup('Label for the group', false)
 * log('info', 'Message for the group')
 * log('warn', 'Field should not be empty')
 * log.end()
 */
function logGroup(label, collapsed = true) {
    const logs = [];
    const end = () => {
        log(collapsed ? "groupCollapsed" : "group", label);
        logs.forEach((messages) => {
            log(...messages);
        });
        log("groupEnd");
    };
    const _log = (type, ...messages) => {
        logs.push([type, ...messages]);
    };
    _log.end = end;
    return _log;
}

function isValidCode(code) {
    return typeof code === "string" && code !== "";
}
function logCode(host, code) {
    if (!isValidCode(code)) {
        log("warn", '"code" attribute should be a not empty string.', host);
    }
}
function isValidQuantity(quantity) {
    return quantity >= 0;
}
function isValidUnion(union, values) {
    return union != null && values.includes(union);
}
function logUnion(host, attributeName, attributeValue, validValues) {
    if (!isValidUnion(attributeValue, validValues)) {
        log("warn", `"${attributeName}" attribute should be one of ${validValues
            .map((v) => `"${v}"`)
            .join(", ")}. Received: "${attributeValue ?? "undefined"}"`, host);
    }
}
/**
 * Converts an Union type to a Tuple
 */
function unionToTuple() {
    return (...elements) => elements;
}

export { logCode as a, logUnion as b, isValidUnion as c, log as d, isValidQuantity as e, isValidCode as i, logGroup as l, unionToTuple as u };
